@@ -1,15 +1,14 @@
-SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, loop_breakers=NULL, unique=FALSE, seed=NULL) {
+SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, afreq=c(0.5, 0.5), loop_breakers=NULL, unique=FALSE, seed=NULL, verbose=TRUE) {
 	if (is.null(x$model)) {
-		if (all(x$pedigree[,'AFF']==1)) {x=setModel(x,1); cat("Unaffected pedigree; assuming autosomal model.\n\n")}
+		if (all(x$pedigree[,'AFF']==1)) {x=setModel(x,1); if(verbose) cat("Unaffected pedigree. Simulating autosomal markers.\n")}
 		else stop("No model set. Use setModel().")
 	}
 	
-	if (is.null(x$model$nallel) || is.null(x$model$afreq)) stop("Incomplete model. Use setModel() to indicate the number of alleles and their frequencies.")
-	if (x$model$nallel>2) stop("Sorry - only diallelic markers allowed in SNP simulation.")
 	if (any(!is.numeric(N), length(N)>1, N%%1 != 0)) stop("N must be a positive integer.")
+	if (length(afreq)>2) stop("Sorry - only diallelic markers allowed in SNP simulation.")
+	nInd = x$nInd; chrom = x$model$chrom; 
 	
-	nInd = x$nInd; chr = x$model$chrom; afreq = x$model$afreq
-	.TRzero <- .TRmatr(0, chr); .TRhalf <- .TRmatr(0.5, chr)
+	.TRzero <- .TRmatr(0, chrom); .TRhalf <- .TRmatr(0.5, chrom)
 	
 	if (is.null(partialmarker)) partialmarker = matrix(0, nrow=nInd, ncol=2) else if(ncol(partialmarker)!=2 || nrow(partialmarker)!=nInd) stop("Wrong dimensions of marker matrix.")
 	x = setMarkers(x, partialmarker, missing=0)
@@ -18,8 +17,10 @@ SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, loop_break
 	
 	m = x$markerdata[[1]]
 	if(length(attr(m, "alleles")[[1]])>2) stop("Partial marker is not diallelic. Only diallelic markers allowed.")
-	cat("Simulating SNPs completely linked to disease locus\n")
-	if(any(m!=0)) { cat("conditional on the following genotypes:\n"); 	print(data.frame(ID=x$orig.ids, GENO=.prettyMarkers(m, missing="-", singleCol=TRUE, chrom=chr, sex=x$pedigree[, 'SEX']))) }
+	if(verbose && any(m!=0)) { 
+		cat("Simulating markers conditional on existing genotypes:\n")
+		print(data.frame(ID=x$orig.ids, GENO=.prettyMarkers(m, missing="-", singleCol=TRUE, sex=x$pedigree[, 'SEX'])), row.names=FALSE)
+	}
 
 	if (loops <- x$hasLoops)	{		
 		if(is.null(loop_breakers)) 	 stop("The pedigree has loops. Please indicate loop breakers.")
@@ -36,19 +37,19 @@ SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, loop_break
 	if (!is.null(seed)) set.seed(seed)
 	
 	# simulate only indivs i) with no given genotype, ii) that are available or have available descendants. Duplicated individuals are not simulated.
-	sim_indivs <- seq_len(nInd)[sapply(seq_len(nInd), function(i) zgeno[i]==0 && (avail[i] || any(avail[descendants(x, i, original.id=F)])))]
+	sim_indivs = seq_len(nInd)[sapply(seq_len(nInd), function(i) zgeno[i]==0 && (avail[i] || any(avail[descendants(x, i, original.id=F)])))]
 	if(length(sim_indivs)==0) stop("Something is wrong: No individuals available for simulation.")
-	cat("Simulating genotypes for the following individuals:\n  ", .prettycat(x$orig.ids[avail & !preexisting], "and"), "\n")
+	if(verbose) cat("Simulating genotypes for the following individuals:", .prettycat(x$orig.ids[avail & !preexisting], "and"), "\n")
 	
 	genoprobs <- function(x, partialgeno, id, values) {  
 			#values are 1:3 for autosomal models, and either 1:2 (males) or 1:3 (females) for X-linked models
 			#outputs vector of length |values| with genotype probs for indiv id given pedigree og partial genotype information
-			probs <- sapply(values, function(g) { partialgeno[id]<-g;   likelihood(x, afreq=afreq, singleNum.geno=partialgeno, TR.MATR=.TRzero) } )
+			probs <- sapply(values, function(g) { partialgeno[id]<-g;   likelihoodSNP(x, afreq=afreq, singleNum.geno=partialgeno, TR.MATR=.TRzero) } )
 			if (sum(probs)==0) { print(cbind(x$pedigree, partialgeno)); stop("\nIndividual ",id,": All genotype probabilities zero. Mendelian error?") }
 			probs
 	}
 	
-	switch(chr,	
+	switch(chrom,	
 	AUTOSOMAL = {
 		# simulation order: founders first (speeds up the likelihoods). 
 		sim_indivs <- sim_indivs[order(sim_indivs %in% x$nonfounders)]
@@ -58,14 +59,14 @@ SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, loop_break
 		initg <- t(.my.grid( rep(list(1:3), init ) ))
 		initp <- apply(initg, 2, function(g) { 
 			zgeno[ sim_indivs[1:init] ] <- g;   
-			likelihood(x, afreq=afreq, singleNum.geno=zgeno, TR.MATR=.TRzero) } )
+			likelihoodSNP(x, afreq=afreq, singleNum.geno=zgeno, TR.MATR=.TRzero) } )
 		if (identical(sum(initp), 0)) stop("All genotype probabilities zero. Wrong model?")
 		
 		# pre-fill the rows of the 'init' individuals (i.e. sim_indivs[1:init]) 
 		markers[sim_indivs[1:init], ] <- initg[ , suppressWarnings(sample.int( ncol(initg), size=N, replace=TRUE, prob=initp )) ]
 
 		# do the rest of the individuals (only those present in sim_indivs)
-		for (i in sim_indivs[-c(1:init)])
+		for (i in sim_indivs[-(1:init)])
 			markers[i, ] <- apply(markers, 2, function(partgeno) sample.int(3, size=1, prob=genoprobs(x, partgeno, id=i, values=1:3)) )
 	},
 	X = {
@@ -81,7 +82,7 @@ SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, loop_break
 		init_indivs = c(males[seq_len(init_m)], females[seq_len(init_f)])
 		# pre-calculate probabilities for the first 'init' individuals
 		initg <- t(.my.grid( list(1:2, 1:3)[rep(1:2, c(init_m, init_f))] ))
-		initp <- apply(initg, 2, function(g) { zgeno[ init_indivs ] <- g;   likelihood(x, afreq=afreq, singleNum.geno=zgeno, TR.MATR=.TRzero) } )
+		initp <- apply(initg, 2, function(g) { zgeno[ init_indivs ] <- g;   likelihoodSNP(x, afreq=afreq, singleNum.geno=zgeno, TR.MATR=.TRzero) } )
 
 		# pre-fill the rows of the 'init_indivs' 
 		markers[init_indivs, ] <- initg[ , suppressWarnings(sample.int( ncol(initg), size=N, replace=TRUE, prob=initp)) ]
@@ -96,8 +97,28 @@ SNPsim <- function(x, N=1, partialmarker=NULL, available=x$available, loop_break
 	
 	markers[!avail & !preexisting, ] <- 0
 	
-	if (unique) markers <- unique(markers, MARGIN=2)
-	x = setMarkers(x, .geno2diallel(markers))
+	if (unique) markers = unique(markers, MARGIN=2)
+	markers2 = .geno2diallel(markers)
+	mlist = lapply(2*seq_len(ncol(markers)), function(i) .createMarkerObject(markers2[, (i-1):i], alleles=1:2, afreq = afreq, missing = 0))
+	x = setMarkers(x, structure(mlist, class = "markerdata"))	
 	if(loops) x = tieLoops(x)
 	x
 }
+
+
+.exclusion = function(x, N, available, id.check, Xchrom=FALSE, afreq=c(0.5, 0.5), return.simdata=FALSE, seed=NULL, verbose=T){
+  if (!is.null(seed)) set.seed(seed)
+  y = simpleSim(x, N=N, afreq=afreq, available=available, Xchrom=Xchrom, verbose=verbose)
+  
+  random.genos = 
+  if(Xchrom) rep(sample.int(length(afreq), N, prob = afreq, replace = TRUE), each=2) 
+  else sample.int(length(afreq), 2*N, prob = afreq, replace = TRUE)
+
+  y = .modifyMarkerData(y, id.check, random.genos)
+  
+  exclude = mendelianCheck(y, verbose=FALSE)
+  exclusion.prob = length(exclude)/N
+  cat("Exclusion probability for individual", id.check, "=", exclusion.prob, "\n")
+  if(return.simdata) return(list(exclusion.prob = exclusion.prob, simdata = y))
+  else return(exclusion.prob)
+}
\ No newline at end of file
